@@ -28,9 +28,11 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 
-	"k8s.io/kubernetes/pkg/api/v1"
-	"k8s.io/kubernetes/pkg/client/clientset_generated/clientset/fake"
-	corelisters "k8s.io/kubernetes/pkg/client/listers/core/v1"
+	"k8s.io/api/core/v1"
+	"k8s.io/client-go/kubernetes/fake"
+	corelisters "k8s.io/client-go/listers/core/v1"
+	_ "k8s.io/kubernetes/pkg/apis/apps/install"
+	_ "k8s.io/kubernetes/pkg/apis/core/install"
 )
 
 func TestStatefulPodControlCreatesPods(t *testing.T) {
@@ -61,7 +63,7 @@ func TestStatefulPodControlCreatesPods(t *testing.T) {
 	}
 	for i := range events {
 		if !strings.Contains(events[i], v1.EventTypeNormal) {
-			t.Errorf("Expected normal events found %s", events[i])
+			t.Errorf("Found unexpected non-normal event %s", events[i])
 		}
 	}
 }
@@ -91,7 +93,7 @@ func TestStatefulPodControlCreatePodExists(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 0 {
-		t.Errorf("Expected 0 events when Pod and PVC exist found %d", eventCount)
+		t.Errorf("Pod and PVC exist: got %d events, but want 0", eventCount)
 		for i := range events {
 			t.Log(events[i])
 		}
@@ -118,11 +120,11 @@ func TestStatefulPodControlCreatePodPvcCreateFailure(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 2 {
-		t.Errorf("Expected 2 events for PVC create failure found %d", eventCount)
+		t.Errorf("PVC create failure: got %d events, but want 2", eventCount)
 	}
 	for i := range events {
 		if !strings.Contains(events[i], v1.EventTypeWarning) {
-			t.Errorf("Expected normal events found %s", events[i])
+			t.Errorf("Found unexpected non-warning event %s", events[i])
 		}
 	}
 }
@@ -156,11 +158,11 @@ func TestStatefulPodControlCreatePodPvcGetFailure(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 2 {
-		t.Errorf("Expected 2 events for PVC create failure found %d", eventCount)
+		t.Errorf("PVC create failure: got %d events, but want 2", eventCount)
 	}
 	for i := range events {
 		if !strings.Contains(events[i], v1.EventTypeWarning) {
-			t.Errorf("Expected normal events found %s", events[i])
+			t.Errorf("Found unexpected non-warning event: %s", events[i])
 		}
 	}
 }
@@ -185,12 +187,12 @@ func TestStatefulPodControlCreatePodFailed(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 2 {
-		t.Errorf("Expected 2 events for failed Pod create found %d", eventCount)
+		t.Errorf("Pod create failed: got %d events, but want 2", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeNormal) {
-		t.Errorf("Expected normal event found %s", events[0])
+		t.Errorf("Found unexpected non-normal event %s", events[0])
 
 	} else if !strings.Contains(events[1], v1.EventTypeWarning) {
-		t.Errorf("Expected warning event found %s", events[1])
+		t.Errorf("Found unexpected non-warning event %s", events[1])
 
 	}
 }
@@ -203,14 +205,14 @@ func TestStatefulPodControlNoOpUpdate(t *testing.T) {
 	control := NewRealStatefulPodControl(fakeClient, nil, nil, nil, recorder)
 	fakeClient.AddReactor("*", "*", func(action core.Action) (bool, runtime.Object, error) {
 		t.Error("no-op update should not make any client invocation")
-		return true, nil, apierrors.NewInternalError(errors.New("If we are here we have a problem"))
+		return true, nil, apierrors.NewInternalError(errors.New("if we are here we have a problem"))
 	})
 	if err := control.UpdateStatefulPod(set, pod); err != nil {
 		t.Errorf("Error returned on no-op update error: %s", err)
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 0 {
-		t.Errorf("Expected 0 events for no-op update found %d", eventCount)
+		t.Errorf("no-op update: got %d events, but want 0", eventCount)
 	}
 }
 
@@ -232,9 +234,9 @@ func TestStatefulPodControlUpdatesIdentity(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 1 {
-		t.Errorf("Expected 1 event for successful Pod update found %d", eventCount)
+		t.Errorf("Pod update successful:got %d events,but want 1", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeNormal) {
-		t.Errorf("Expected normal event found %s", events[0])
+		t.Errorf("Found unexpected non-normal event %s", events[0])
 	}
 	if !identityMatches(set, updated) {
 		t.Error("Name update failed identity does not match")
@@ -262,9 +264,9 @@ func TestStatefulPodControlUpdateIdentityFailure(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 1 {
-		t.Errorf("Expected 1 event for failed Pod update found %d", eventCount)
+		t.Errorf("Pod update failed: got %d events, but want 1", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeWarning) {
-		t.Errorf("Expected warning event found %s", events[0])
+		t.Errorf("Found unexpected non-warning event %s", events[0])
 	}
 	if identityMatches(set, pod) {
 		t.Error("Failed update mutated Pod identity")
@@ -280,7 +282,7 @@ func TestStatefulPodControlUpdatesPodStorage(t *testing.T) {
 	pvcLister := corelisters.NewPersistentVolumeClaimLister(pvcIndexer)
 	control := NewRealStatefulPodControl(fakeClient, nil, nil, pvcLister, recorder)
 	pvcs := getPersistentVolumeClaims(set, pod)
-	volumes := make([]v1.Volume, len(pod.Spec.Volumes))
+	volumes := make([]v1.Volume, 0, len(pod.Spec.Volumes))
 	for i := range pod.Spec.Volumes {
 		if _, contains := pvcs[pod.Spec.Volumes[i].Name]; !contains {
 			volumes = append(volumes, pod.Spec.Volumes[i])
@@ -306,11 +308,11 @@ func TestStatefulPodControlUpdatesPodStorage(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 2 {
-		t.Errorf("Expected 2 event for successful Pod storage update found %d", eventCount)
+		t.Errorf("Pod storage update successful: got %d events, but want 2", eventCount)
 	}
 	for i := range events {
 		if !strings.Contains(events[i], v1.EventTypeNormal) {
-			t.Errorf("Expected normal event found %s", events[i])
+			t.Errorf("Found unexpected non-normal event %s", events[i])
 		}
 	}
 	if !storageMatches(set, updated) {
@@ -327,7 +329,7 @@ func TestStatefulPodControlUpdatePodStorageFailure(t *testing.T) {
 	pvcLister := corelisters.NewPersistentVolumeClaimLister(pvcIndexer)
 	control := NewRealStatefulPodControl(fakeClient, nil, nil, pvcLister, recorder)
 	pvcs := getPersistentVolumeClaims(set, pod)
-	volumes := make([]v1.Volume, len(pod.Spec.Volumes))
+	volumes := make([]v1.Volume, 0, len(pod.Spec.Volumes))
 	for i := range pod.Spec.Volumes {
 		if _, contains := pvcs[pod.Spec.Volumes[i].Name]; !contains {
 			volumes = append(volumes, pod.Spec.Volumes[i])
@@ -346,11 +348,11 @@ func TestStatefulPodControlUpdatePodStorageFailure(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 2 {
-		t.Errorf("Expected 2 event for failed Pod storage update found %d", eventCount)
+		t.Errorf("Pod storage update failed: got %d events, but want 2", eventCount)
 	}
 	for i := range events {
 		if !strings.Contains(events[i], v1.EventTypeWarning) {
-			t.Errorf("Expected normal event found %s", events[i])
+			t.Errorf("Found unexpected non-normal event %s", events[i])
 		}
 	}
 }
@@ -372,9 +374,9 @@ func TestStatefulPodControlUpdatePodConflictSuccess(t *testing.T) {
 		if !conflict {
 			conflict = true
 			return true, update.GetObject(), apierrors.NewConflict(action.GetResource().GroupResource(), pod.Name, errors.New("conflict"))
-		} else {
-			return true, update.GetObject(), nil
 		}
+		return true, update.GetObject(), nil
+
 	})
 	pod.Name = "goo-0"
 	if err := control.UpdateStatefulPod(set, pod); err != nil {
@@ -382,9 +384,9 @@ func TestStatefulPodControlUpdatePodConflictSuccess(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 1 {
-		t.Errorf("Expected 1 event for successful Pod update found %d", eventCount)
+		t.Errorf("Pod update successful: got %d, but want 1", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeNormal) {
-		t.Errorf("Expected normal event found %s", events[0])
+		t.Errorf("Found unexpected non-normal event %s", events[0])
 	}
 	if !identityMatches(set, pod) {
 		t.Error("Name update failed identity does not match")
@@ -405,9 +407,9 @@ func TestStatefulPodControlDeletesStatefulPod(t *testing.T) {
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 1 {
-		t.Errorf("Expected 1 events for successful delete found %d", eventCount)
+		t.Errorf("delete successful: got %d events, but want 1", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeNormal) {
-		t.Errorf("Expected normal event found %s", events[0])
+		t.Errorf("Found unexpected non-normal event %s", events[0])
 	}
 }
 
@@ -421,13 +423,13 @@ func TestStatefulPodControlDeleteFailure(t *testing.T) {
 		return true, nil, apierrors.NewInternalError(errors.New("API server down"))
 	})
 	if err := control.DeleteStatefulPod(set, pod); err == nil {
-		t.Error("Fialed to return error on failed delete")
+		t.Error("Failed to return error on failed delete")
 	}
 	events := collectEvents(recorder.Events)
 	if eventCount := len(events); eventCount != 1 {
-		t.Errorf("Expected 1 events for failed delete found %d", eventCount)
+		t.Errorf("delete failed: got %d events, but want 1", eventCount)
 	} else if !strings.Contains(events[0], v1.EventTypeWarning) {
-		t.Errorf("Expected warning event found %s", events[0])
+		t.Errorf("Found unexpected non-warning event %s", events[0])
 	}
 }
 
