@@ -27,8 +27,10 @@ import (
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes/fake"
+	"k8s.io/kubernetes/pkg/scheduler/apis/config"
 	framework "k8s.io/kubernetes/pkg/scheduler/framework/v1alpha1"
 	"k8s.io/kubernetes/pkg/scheduler/internal/cache"
+	"k8s.io/kubernetes/pkg/scheduler/internal/parallelize"
 	st "k8s.io/kubernetes/pkg/scheduler/testing"
 	"k8s.io/utils/pointer"
 )
@@ -46,12 +48,12 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 			name: "normal case",
 			pod: st.MakePod().Name("p").Label("foo", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			want: &preScoreState{
 				Constraints: []topologySpreadConstraint{
@@ -62,30 +64,28 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 					},
 					{
 						MaxSkew:     1,
-						TopologyKey: "node",
+						TopologyKey: v1.LabelHostname,
 						Selector:    mustConvertLabelSelectorAsSelector(t, st.MakeLabelSelector().Exists("foo").Obj()),
 					},
 				},
-				NodeNameSet: sets.NewString("node-a", "node-b", "node-x"),
+				IgnoredNodes: sets.NewString(),
 				TopologyPairToPodCounts: map[topologyPair]*int64{
-					{key: "zone", value: "zone1"}:  pointer.Int64Ptr(0),
-					{key: "zone", value: "zone2"}:  pointer.Int64Ptr(0),
-					{key: "node", value: "node-a"}: pointer.Int64Ptr(0),
-					{key: "node", value: "node-b"}: pointer.Int64Ptr(0),
-					{key: "node", value: "node-x"}: pointer.Int64Ptr(0),
+					{key: "zone", value: "zone1"}: pointer.Int64Ptr(0),
+					{key: "zone", value: "zone2"}: pointer.Int64Ptr(0),
 				},
+				TopologyNormalizingWeight: []float64{topologyNormalizingWeight(2), topologyNormalizingWeight(3)},
 			},
 		},
 		{
 			name: "node-x doesn't have label zone",
 			pod: st.MakePod().Name("p").Label("foo", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
 				Obj(),
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			want: &preScoreState{
 				Constraints: []topologySpreadConstraint{
@@ -96,28 +96,27 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 					},
 					{
 						MaxSkew:     1,
-						TopologyKey: "node",
+						TopologyKey: v1.LabelHostname,
 						Selector:    mustConvertLabelSelectorAsSelector(t, st.MakeLabelSelector().Exists("bar").Obj()),
 					},
 				},
-				NodeNameSet: sets.NewString("node-a", "node-b"),
+				IgnoredNodes: sets.NewString("node-x"),
 				TopologyPairToPodCounts: map[topologyPair]*int64{
-					{key: "zone", value: "zone1"}:  pointer.Int64Ptr(0),
-					{key: "node", value: "node-a"}: pointer.Int64Ptr(0),
-					{key: "node", value: "node-b"}: pointer.Int64Ptr(0),
+					{key: "zone", value: "zone1"}: pointer.Int64Ptr(0),
 				},
+				TopologyNormalizingWeight: []float64{topologyNormalizingWeight(1), topologyNormalizingWeight(2)},
 			},
 		},
 		{
 			name: "defaults constraints and a replica set",
 			pod:  st.MakePod().Name("p").Label("foo", "tar").Label("baz", "sup").Obj(),
 			defaultConstraints: []v1.TopologySpreadConstraint{
-				{MaxSkew: 1, TopologyKey: "node", WhenUnsatisfiable: v1.ScheduleAnyway},
+				{MaxSkew: 1, TopologyKey: v1.LabelHostname, WhenUnsatisfiable: v1.ScheduleAnyway},
 				{MaxSkew: 2, TopologyKey: "rack", WhenUnsatisfiable: v1.DoNotSchedule},
 				{MaxSkew: 2, TopologyKey: "planet", WhenUnsatisfiable: v1.ScheduleAnyway},
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("rack", "rack1").Label("node", "node-a").Label("planet", "mars").Obj(),
+				st.MakeNode().Name("node-a").Label("rack", "rack1").Label(v1.LabelHostname, "node-a").Label("planet", "mars").Obj(),
 			},
 			objs: []runtime.Object{
 				&appsv1.ReplicaSet{Spec: appsv1.ReplicaSetSpec{Selector: st.MakeLabelSelector().Exists("foo").Obj()}},
@@ -126,7 +125,7 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 				Constraints: []topologySpreadConstraint{
 					{
 						MaxSkew:     1,
-						TopologyKey: "node",
+						TopologyKey: v1.LabelHostname,
 						Selector:    mustConvertLabelSelectorAsSelector(t, st.MakeLabelSelector().Exists("foo").Obj()),
 					},
 					{
@@ -135,11 +134,11 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 						Selector:    mustConvertLabelSelectorAsSelector(t, st.MakeLabelSelector().Exists("foo").Obj()),
 					},
 				},
-				NodeNameSet: sets.NewString("node-a"),
+				IgnoredNodes: sets.NewString(),
 				TopologyPairToPodCounts: map[topologyPair]*int64{
-					{key: "node", value: "node-a"}: pointer.Int64Ptr(0),
 					{key: "planet", value: "mars"}: pointer.Int64Ptr(0),
 				},
+				TopologyNormalizingWeight: []float64{topologyNormalizingWeight(1), topologyNormalizingWeight(1)},
 			},
 		},
 		{
@@ -180,10 +179,11 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 						Selector:    mustConvertLabelSelectorAsSelector(t, st.MakeLabelSelector().Label("baz", "sup").Obj()),
 					},
 				},
-				NodeNameSet: sets.NewString("node-a"),
+				IgnoredNodes: sets.NewString(),
 				TopologyPairToPodCounts: map[topologyPair]*int64{
 					{"planet", "mars"}: pointer.Int64Ptr(0),
 				},
+				TopologyNormalizingWeight: []float64{topologyNormalizingWeight(1)},
 			},
 		},
 	}
@@ -193,7 +193,7 @@ func TestPreScoreStateEmptyNodes(t *testing.T) {
 			informerFactory := informers.NewSharedInformerFactory(fake.NewSimpleClientset(tt.objs...), 0)
 			pl := PodTopologySpread{
 				sharedLister: cache.NewSnapshot(nil, tt.nodes),
-				Args: Args{
+				args: config.PodTopologySpreadArgs{
 					DefaultConstraints: tt.defaultConstraints,
 				},
 			}
@@ -233,14 +233,13 @@ func TestPodTopologySpreadScore(t *testing.T) {
 		//    but node2 either i) doesn't have all required topologyKeys present, or ii) doesn't match
 		//    incoming pod's nodeSelector/nodeAffinity
 		{
-			// if there is only one candidate node, it should be scored to 10
 			name: "one constraint on node, no existing pods",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
 			},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
@@ -248,10 +247,10 @@ func TestPodTopologySpreadScore(t *testing.T) {
 			},
 		},
 		{
-			// if there is only one candidate node, it should be scored to 10
+			// if there is only one candidate node, it should be scored to 100
 			name: "one constraint on node, only one node is candidate",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -259,10 +258,10 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-b1").Node("node-b").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
 			},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
@@ -271,15 +270,15 @@ func TestPodTopologySpreadScore(t *testing.T) {
 		{
 			name: "one constraint on node, all nodes have the same number of matching pods",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
 				st.MakePod().Name("p-b1").Node("node-b").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
 			},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
@@ -287,12 +286,10 @@ func TestPodTopologySpreadScore(t *testing.T) {
 			},
 		},
 		{
-			// matching pods spread as 2/1/0/3, total = 6
-			// after reversing, it's 4/5/6/3
-			// so scores = 400/6, 500/6, 600/6, 300/6
+			// matching pods spread as 2/1/0/3.
 			name: "one constraint on node, all 4 nodes are candidates",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -303,26 +300,84 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-d3").Node("node-d").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-c").Label("node", "node-c").Obj(),
-				st.MakeNode().Name("node-d").Label("node", "node-d").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-c").Label(v1.LabelHostname, "node-c").Obj(),
+				st.MakeNode().Name("node-d").Label(v1.LabelHostname, "node-d").Obj(),
 			},
 			failedNodes: []*v1.Node{},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 66},
-				{Name: "node-b", Score: 83},
+				{Name: "node-a", Score: 40},
+				{Name: "node-b", Score: 80},
 				{Name: "node-c", Score: 100},
-				{Name: "node-d", Score: 50},
+				{Name: "node-d", Score: 0},
 			},
 		},
 		{
-			// matching pods spread as 4/2/1/~3~, total = 4+2+1 = 7 (as node4 is not a candidate)
-			// after reversing, it's 3/5/6
-			// so scores = 300/6, 500/6, 600/6
+			name: "one constraint on node, all 4 nodes are candidates, maxSkew=2",
+			pod: st.MakePod().Name("p").Label("foo", "").
+				SpreadConstraint(2, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				Obj(),
+			// matching pods spread as 2/1/0/3.
+			existingPods: []*v1.Pod{
+				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-a2").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-b1").Node("node-b").Label("foo", "").Obj(),
+				st.MakePod().Name("p-d1").Node("node-d").Label("foo", "").Obj(),
+				st.MakePod().Name("p-d2").Node("node-d").Label("foo", "").Obj(),
+				st.MakePod().Name("p-d3").Node("node-d").Label("foo", "").Obj(),
+			},
+			nodes: []*v1.Node{
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-c").Label(v1.LabelHostname, "node-c").Obj(),
+				st.MakeNode().Name("node-d").Label(v1.LabelHostname, "node-d").Obj(),
+			},
+			failedNodes: []*v1.Node{},
+			want: []framework.NodeScore{
+				{Name: "node-a", Score: 50}, // +10, compared to maxSkew=1
+				{Name: "node-b", Score: 83}, // +3, compared to maxSkew=1
+				{Name: "node-c", Score: 100},
+				{Name: "node-d", Score: 16}, // +16, compared to maxSkew=1
+			},
+		},
+		{
+			name: "one constraint on node, all 4 nodes are candidates, maxSkew=3",
+			pod: st.MakePod().Name("p").Label("foo", "").
+				SpreadConstraint(3, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				Obj(),
+			existingPods: []*v1.Pod{
+				// matching pods spread as 4/3/2/1.
+				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-a2").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-a3").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-a4").Node("node-a").Label("foo", "").Obj(),
+				st.MakePod().Name("p-b1").Node("node-b").Label("foo", "").Obj(),
+				st.MakePod().Name("p-b2").Node("node-b").Label("foo", "").Obj(),
+				st.MakePod().Name("p-b3").Node("node-b").Label("foo", "").Obj(),
+				st.MakePod().Name("p-c1").Node("node-c").Label("foo", "").Obj(),
+				st.MakePod().Name("p-c2").Node("node-c").Label("foo", "").Obj(),
+				st.MakePod().Name("p-d1").Node("node-d").Label("foo", "").Obj(),
+			},
+			nodes: []*v1.Node{
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-c").Label(v1.LabelHostname, "node-c").Obj(),
+				st.MakeNode().Name("node-d").Label(v1.LabelHostname, "node-d").Obj(),
+			},
+			failedNodes: []*v1.Node{},
+			want: []framework.NodeScore{
+				{Name: "node-a", Score: 33}, // +19 compared to maxSkew=1
+				{Name: "node-b", Score: 55}, // +13 compared to maxSkew=1
+				{Name: "node-c", Score: 77}, // +6 compared to maxSkew=1
+				{Name: "node-d", Score: 100},
+			},
+		},
+		{
+			// matching pods spread as 4/2/1/~3~ (node4 is not a candidate)
 			name: "one constraint on node, 3 out of 4 nodes are candidates",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -337,26 +392,24 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y3").Node("node-y").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-y").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-y").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 50},
-				{Name: "node-b", Score: 83},
+				{Name: "node-a", Score: 16},
+				{Name: "node-b", Score: 66},
 				{Name: "node-x", Score: 100},
 			},
 		},
 		{
 			// matching pods spread as 4/?2?/1/~3~, total = 4+?+1 = 5 (as node2 is problematic)
-			// after reversing, it's 1/?/4
-			// so scores = 100/4, 0, 400/4
-			name: "one constraint on node, 3 out of 4 nodes are candidates",
+			name: "one constraint on node, 3 out of 4 nodes are candidates, one node doesn't match topology key",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -371,23 +424,21 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y3").Node("node-y").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
 				st.MakeNode().Name("node-b").Label("n", "node-b").Obj(), // label `n` doesn't match topologyKey
-				st.MakeNode().Name("node-x").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-x").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-y").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-y").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 25},
+				{Name: "node-a", Score: 20},
 				{Name: "node-b", Score: 0},
 				{Name: "node-x", Score: 100},
 			},
 		},
 		{
-			// matching pods spread as 4/2/1/~3~, total = 6+6+4 = 16 (as topologyKey is zone instead of node)
-			// after reversing, it's 10/10/12
-			// so scores = 1000/12, 1000/12, 1200/12
+			// matching pods spread as 4/2/1/~3~
 			name: "one constraint on zone, 3 out of 4 nodes are candidates",
 			pod: st.MakePod().Name("p").Label("foo", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
@@ -405,27 +456,25 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y3").Node("node-y").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-y").Label("zone", "zone2").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-y").Label("zone", "zone2").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 83},
-				{Name: "node-b", Score: 83},
+				{Name: "node-a", Score: 62},
+				{Name: "node-b", Score: 62},
 				{Name: "node-x", Score: 100},
 			},
 		},
 		{
-			// matching pods spread as 2/~1~/2/~4~, total = 2+3 + 2+6 = 13 (zone and node should be both summed up)
-			// after reversing, it's 8/5
-			// so scores = 800/8, 500/8
+			// matching pods spread as 2/~1~/2/~4~.
 			name: "two Constraints on zone and node, 2 out of 4 nodes are candidates",
 			pod: st.MakePod().Name("p").Label("foo", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -439,16 +488,16 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y4").Node("node-y").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-y").Label("zone", "zone2").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-y").Label("zone", "zone2").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
-				{Name: "node-x", Score: 62},
+				{Name: "node-x", Score: 54},
 			},
 		},
 		{
@@ -462,13 +511,10 @@ func TestPodTopologySpreadScore(t *testing.T) {
 			// +--------+-------------+--------+---------------+
 			// For the first constraint (zone): the matching pods spread as 2/2/1/1
 			// For the second constraint (node): the matching pods spread as 0/1/0/1
-			// sum them up gets: 2/3/1/2, and total number is 8.
-			// after reversing, it's 6/5/7/6
-			// so scores = 600/7, 500/7, 700/7, 600/7
 			name: "two Constraints on zone and node, with different labelSelectors",
 			pod: st.MakePod().Name("p").Label("foo", "").Label("bar", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -477,29 +523,26 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y2").Node("node-y").Label("bar", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
-				st.MakeNode().Name("node-y").Label("zone", "zone2").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
+				st.MakeNode().Name("node-y").Label("zone", "zone2").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			failedNodes: []*v1.Node{},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 85},
-				{Name: "node-b", Score: 71},
+				{Name: "node-a", Score: 75},
+				{Name: "node-b", Score: 25},
 				{Name: "node-x", Score: 100},
-				{Name: "node-y", Score: 85},
+				{Name: "node-y", Score: 50},
 			},
 		},
 		{
 			// For the first constraint (zone): the matching pods spread as 0/0/2/2
 			// For the second constraint (node): the matching pods spread as 0/1/0/1
-			// sum them up gets: 0/1/2/3, and total number is 6.
-			// after reversing, it's 6/5/4/3.
-			// so scores = 600/6, 500/6, 400/6, 300/6
 			name: "two Constraints on zone and node, with different labelSelectors, some nodes have 0 pods",
 			pod: st.MakePod().Name("p").Label("foo", "").Label("bar", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-b1").Node("node-b").Label("bar", "").Obj(),
@@ -507,29 +550,26 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y1").Node("node-y").Label("foo", "").Label("bar", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
-				st.MakeNode().Name("node-y").Label("zone", "zone2").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
+				st.MakeNode().Name("node-y").Label("zone", "zone2").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			failedNodes: []*v1.Node{},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
-				{Name: "node-b", Score: 83},
-				{Name: "node-x", Score: 66},
-				{Name: "node-y", Score: 50},
+				{Name: "node-b", Score: 75},
+				{Name: "node-x", Score: 50},
+				{Name: "node-y", Score: 0},
 			},
 		},
 		{
 			// For the first constraint (zone): the matching pods spread as 2/2/1/~1~
 			// For the second constraint (node): the matching pods spread as 0/1/0/~1~
-			// sum them up gets: 2/3/1, and total number is 6.
-			// after reversing, it's 4/3/5
-			// so scores = 400/5, 300/5, 500/5
 			name: "two Constraints on zone and node, with different labelSelectors, 3 out of 4 nodes are candidates",
 			pod: st.MakePod().Name("p").Label("foo", "").Label("bar", "").
 				SpreadConstraint(1, "zone", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("bar").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Node("node-a").Label("foo", "").Obj(),
@@ -538,23 +578,23 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-y2").Node("node-y").Label("bar", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("zone", "zone1").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("zone", "zone1").Label("node", "node-b").Obj(),
-				st.MakeNode().Name("node-x").Label("zone", "zone2").Label("node", "node-x").Obj(),
+				st.MakeNode().Name("node-a").Label("zone", "zone1").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label("zone", "zone1").Label(v1.LabelHostname, "node-b").Obj(),
+				st.MakeNode().Name("node-x").Label("zone", "zone2").Label(v1.LabelHostname, "node-x").Obj(),
 			},
 			failedNodes: []*v1.Node{
-				st.MakeNode().Name("node-y").Label("zone", "zone2").Label("node", "node-y").Obj(),
+				st.MakeNode().Name("node-y").Label("zone", "zone2").Label(v1.LabelHostname, "node-y").Obj(),
 			},
 			want: []framework.NodeScore{
-				{Name: "node-a", Score: 80},
-				{Name: "node-b", Score: 60},
+				{Name: "node-a", Score: 75},
+				{Name: "node-b", Score: 25},
 				{Name: "node-x", Score: 100},
 			},
 		},
 		{
 			name: "existing pods in a different namespace do not count",
 			pod: st.MakePod().Name("p").Label("foo", "").
-				SpreadConstraint(1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
+				SpreadConstraint(1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj()).
 				Obj(),
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a1").Namespace("ns1").Node("node-a").Label("foo", "").Obj(),
@@ -563,8 +603,8 @@ func TestPodTopologySpreadScore(t *testing.T) {
 				st.MakePod().Name("p-b2").Node("node-b").Label("foo", "").Obj(),
 			},
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
 			},
 			want: []framework.NodeScore{
 				{Name: "node-a", Score: 100},
@@ -574,11 +614,11 @@ func TestPodTopologySpreadScore(t *testing.T) {
 		{
 			name: "terminating Pods should be excluded",
 			pod: st.MakePod().Name("p").Label("foo", "").SpreadConstraint(
-				1, "node", v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj(),
+				1, v1.LabelHostname, v1.ScheduleAnyway, st.MakeLabelSelector().Exists("foo").Obj(),
 			).Obj(),
 			nodes: []*v1.Node{
-				st.MakeNode().Name("node-a").Label("node", "node-a").Obj(),
-				st.MakeNode().Name("node-b").Label("node", "node-b").Obj(),
+				st.MakeNode().Name("node-a").Label(v1.LabelHostname, "node-a").Obj(),
+				st.MakeNode().Name("node-b").Label(v1.LabelHostname, "node-b").Obj(),
 			},
 			existingPods: []*v1.Pod{
 				st.MakePod().Name("p-a").Node("node-a").Label("foo", "").Terminating().Obj(),
@@ -695,8 +735,8 @@ func BenchmarkTestPodTopologySpreadScore(b *testing.B) {
 }
 
 // The following test allows to compare PodTopologySpread.Score with
-// DefaultPodTopologySpread.Score by using a similar rule.
-// See pkg/scheduler/framework/plugins/defaultpodtopologyspread/default_pod_topology_spread_perf_test.go
+// SelectorSpread.Score by using a similar rule.
+// See pkg/scheduler/framework/plugins/selectorspread/selector_spread_perf_test.go
 // for the equivalent test.
 
 var (
@@ -727,7 +767,7 @@ func BenchmarkTestDefaultEvenPodsSpreadPriority(b *testing.B) {
 			snapshot := cache.NewSnapshot(existingPods, allNodes)
 			p := &PodTopologySpread{
 				sharedLister: snapshot,
-				Args: Args{
+				args: config.PodTopologySpreadArgs{
 					DefaultConstraints: []v1.TopologySpreadConstraint{
 						{MaxSkew: 1, TopologyKey: v1.LabelHostname, WhenUnsatisfiable: v1.ScheduleAnyway},
 						{MaxSkew: 1, TopologyKey: v1.LabelZoneFailureDomain, WhenUnsatisfiable: v1.ScheduleAnyway},
@@ -746,19 +786,18 @@ func BenchmarkTestDefaultEvenPodsSpreadPriority(b *testing.B) {
 			b.ResetTimer()
 
 			for i := 0; i < b.N; i++ {
-				var gotList framework.NodeScoreList
 				status := p.PreScore(ctx, state, pod, filteredNodes)
 				if !status.IsSuccess() {
 					b.Fatalf("unexpected error: %v", status)
 				}
-				for _, n := range filteredNodes {
-					score, status := p.Score(context.Background(), state, pod, n.Name)
-					if !status.IsSuccess() {
-						b.Fatalf("unexpected error: %v", status)
-					}
-					gotList = append(gotList, framework.NodeScore{Name: n.Name, Score: score})
+				gotList := make(framework.NodeScoreList, len(filteredNodes))
+				scoreNode := func(i int) {
+					n := filteredNodes[i]
+					score, _ := p.Score(ctx, state, pod, n.Name)
+					gotList[i] = framework.NodeScore{Name: n.Name, Score: score}
 				}
-				status = p.NormalizeScore(context.Background(), state, pod, gotList)
+				parallelize.Until(ctx, len(filteredNodes), scoreNode)
+				status = p.NormalizeScore(ctx, state, pod, gotList)
 				if !status.IsSuccess() {
 					b.Fatal(status)
 				}
